@@ -1,8 +1,8 @@
-{-# LANGUAGE CPP #-}
 {-# LANGUAGE QuasiQuotes #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE CPP #-}
 module Yesod.Form.Functions
     ( -- * Running in MForm monad
       newFormIdent
@@ -18,7 +18,7 @@ module Yesod.Form.Functions
     , aopt
       -- * Run a form
     , runFormPost
-    , runFormPostNoNonce
+    , runFormPostNoToken
     , runFormGet
       -- * Generate a blank form
     , generateFormPost
@@ -44,11 +44,17 @@ import Control.Arrow (second)
 import Control.Monad.Trans.RWS (ask, get, put, runRWST, tell, evalRWST)
 import Control.Monad.Trans.Class (lift)
 import Control.Monad (liftM, join)
+#if MIN_VERSION_blaze_html(0, 5, 0)
+import Text.Blaze (Markup, toMarkup)
+#define Html Markup
+#define toHtml toMarkup
+#else
 import Text.Blaze (Html, toHtml)
+#endif
 import Yesod.Handler (GHandler, getRequest, runRequestBody, newIdent, getYesod)
 import Yesod.Core (RenderMessage, SomeMessage (..))
 import Yesod.Widget (GWidget, whamlet)
-import Yesod.Request (reqNonce, reqWaiRequest, reqGetParams, languages, FileInfo (..))
+import Yesod.Request (reqToken, reqWaiRequest, reqGetParams, languages, FileInfo (..))
 import Network.Wai (requestMethod)
 import Text.Hamlet (shamlet)
 import Data.Monoid (mempty)
@@ -56,14 +62,8 @@ import Data.Maybe (listToMaybe, fromMaybe)
 import Yesod.Message (RenderMessage (..))
 import qualified Data.Map as Map
 import qualified Data.ByteString.Lazy as L
-
-#if __GLASGOW_HASKELL__ >= 700
-#define WHAMLET whamlet
-#define HTML shamlet
-#else
-#define HTML $shamlet
-#define WHAMLET $whamlet
-#endif
+import Control.Applicative ((<$>))
+import Control.Arrow (first)
 
 -- | Get a unique identifier.
 newFormIdent :: MForm sub master Text
@@ -100,19 +100,17 @@ askFiles = do
     (x, _, _) <- ask
     return $ liftM snd x
 
-mreq :: (RenderMessage master msg, RenderMessage master FormMessage)
-     => Field sub master a -> FieldSettings msg -> Maybe a
+mreq :: RenderMessage master FormMessage
+     => Field sub master a -> FieldSettings master -> Maybe a
      -> MForm sub master (FormResult a, FieldView sub master)
 mreq field fs mdef = mhelper field fs mdef (\m l -> FormFailure [renderMessage m l MsgValueRequired]) FormSuccess True
 
-mopt :: RenderMessage master msg
-     => Field sub master a -> FieldSettings msg -> Maybe (Maybe a)
+mopt :: Field sub master a -> FieldSettings master -> Maybe (Maybe a)
      -> MForm sub master (FormResult (Maybe a), FieldView sub master)
 mopt field fs mdef = mhelper field fs (join mdef) (const $ const $ FormSuccess Nothing) (FormSuccess . Just) False
 
-mhelper :: RenderMessage master msg
-        => Field sub master a
-        -> FieldSettings msg
+mhelper :: Field sub master a
+        -> FieldSettings master
         -> Maybe a
         -> (master -> [Text] -> FormResult b) -- ^ on missing
         -> (a -> FormResult b) -- ^ on success
@@ -141,7 +139,7 @@ mhelper Field {..} FieldSettings {..} mdef onMissing onFound isReq = do
         { fvLabel = toHtml $ mr2 fsLabel
         , fvTooltip = fmap toHtml $ fmap mr2 fsTooltip
         , fvId = theId
-        , fvInput = fieldView theId name fsClass val isReq
+        , fvInput = fieldView theId name fsAttrs val isReq
         , fvErrors =
             case res of
                 FormFailure [e] -> Just $ toHtml e
@@ -149,14 +147,13 @@ mhelper Field {..} FieldSettings {..} mdef onMissing onFound isReq = do
         , fvRequired = isReq
         })
 
-areq :: (RenderMessage master msg, RenderMessage master FormMessage)
-     => Field sub master a -> FieldSettings msg -> Maybe a
+areq :: RenderMessage master FormMessage
+     => Field sub master a -> FieldSettings master -> Maybe a
      -> AForm sub master a
 areq a b = formToAForm . fmap (second return) . mreq a b
 
-aopt :: RenderMessage master msg
-     => Field sub master a
-     -> FieldSettings msg
+aopt :: Field sub master a
+     -> FieldSettings master
      -> Maybe (Maybe a)
      -> AForm sub master (Maybe a)
 aopt a b = formToAForm . fmap (second return) . mopt a b
@@ -186,18 +183,18 @@ postHelper  :: RenderMessage master FormMessage
             -> GHandler sub master ((FormResult a, xml), Enctype)
 postHelper form env = do
     req <- getRequest
-    let nonceKey = "_nonce"
-    let nonce =
-            case reqNonce req of
+    let tokenKey = "_token"
+    let token =
+            case reqToken req of
                 Nothing -> mempty
-                Just n -> [HTML|<input type=hidden name=#{nonceKey} value=#{n}>|]
+                Just n -> [shamlet|<input type=hidden name=#{tokenKey} value=#{n}>|]
     m <- getYesod
     langs <- languages
-    ((res, xml), enctype) <- runFormGeneric (form nonce) m langs env
+    ((res, xml), enctype) <- runFormGeneric (form token) m langs env
     let res' =
             case (res, env) of
                 (FormSuccess{}, Just (params, _))
-                    | Map.lookup nonceKey params /= fmap return (reqNonce req) ->
+                    | Map.lookup tokenKey params /= fmap return (reqToken req) ->
                         FormFailure [renderMessage m langs MsgCsrfWarning]
                 _ -> res
     return ((res', xml), enctype)
@@ -209,8 +206,8 @@ postHelper form env = do
 generateFormPost
     :: RenderMessage master FormMessage
     => (Html -> MForm sub master (FormResult a, xml))
-    -> GHandler sub master ((FormResult a, xml), Enctype)
-generateFormPost form = postHelper form Nothing
+    -> GHandler sub master (xml, Enctype)
+generateFormPost form = first snd <$> postHelper form Nothing
 
 postEnv :: GHandler sub master (Maybe (Env, FileEnv))
 postEnv = do
@@ -224,8 +221,8 @@ postEnv = do
   where
     notEmpty = not . L.null . fileContent
 
-runFormPostNoNonce :: (Html -> MForm sub master (FormResult a, xml)) -> GHandler sub master ((FormResult a, xml), Enctype)
-runFormPostNoNonce form = do
+runFormPostNoToken :: (Html -> MForm sub master (FormResult a, xml)) -> GHandler sub master ((FormResult a, xml), Enctype)
+runFormPostNoToken form = do
     langs <- languages
     m <- getYesod
     env <- postEnv
@@ -248,7 +245,7 @@ getKey = "_hasdata"
 
 getHelper :: (Html -> MForm sub master a) -> Maybe (Env, FileEnv) -> GHandler sub master (a, Enctype)
 getHelper form env = do
-    let fragment = [HTML|<input type=hidden name=#{getKey}>|]
+    let fragment = [shamlet|<input type=hidden name=#{getKey}>|]
     langs <- languages
     m <- getYesod
     runFormGeneric (form fragment) m langs env
@@ -263,7 +260,7 @@ renderTable aform fragment = do
     (res, views') <- aFormToForm aform
     let views = views' []
     -- FIXME non-valid HTML
-    let widget = [WHAMLET|
+    let widget = [whamlet|
 \#{fragment}
 $forall view <- views
     <tr :fvRequired view:.required :not $ fvRequired view:.optional>
@@ -280,7 +277,7 @@ $forall view <- views
 renderDivs aform fragment = do
     (res, views') <- aFormToForm aform
     let views = views' []
-    let widget = [WHAMLET|
+    let widget = [whamlet|
 \#{fragment}
 $forall view <- views
     <div :fvRequired view:.required :not $ fvRequired view:.optional>
@@ -293,7 +290,7 @@ $forall view <- views
 |]
     return (res, widget)
 
--- | Render a form using Bootstrap-friendly HTML syntax.
+-- | Render a form using Bootstrap-friendly shamlet syntax.
 --
 -- Sample Hamlet:
 --
@@ -356,8 +353,8 @@ customErrorMessage msg field = field { fieldParse = \ts -> fmap (either
 (const $ Left msg) Right) $ fieldParse field ts }
 
 -- | Generate a 'FieldSettings' from the given label.
-fieldSettingsLabel :: msg -> FieldSettings msg
-fieldSettingsLabel msg = FieldSettings msg Nothing Nothing Nothing []
+fieldSettingsLabel :: RenderMessage master msg => msg -> FieldSettings master
+fieldSettingsLabel msg = FieldSettings (SomeMessage msg) Nothing Nothing Nothing []
 
 -- | Generate an 'AForm' that gets its value from the given action.
 aformM :: GHandler sub master a -> AForm sub master a
