@@ -1,6 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE ExistentialQuantification #-}
+{-# LANGUAGE CPP #-}
 module Yesod.Form.Types
     ( -- * Helpers
       Enctype (..)
@@ -22,7 +23,14 @@ import Control.Monad.Trans.RWS (RWST)
 import Yesod.Request (FileInfo)
 import Data.Text (Text)
 import Data.Monoid (Monoid (..))
+#if MIN_VERSION_blaze_html(0, 5, 0)
+import Text.Blaze (Markup, ToMarkup (toMarkup))
+#define Html Markup
+#define ToHtml ToMarkup
+#define toHtml toMarkup
+#else
 import Text.Blaze (Html, ToHtml (toHtml))
+#endif
 import Control.Applicative ((<$>), Applicative (..))
 import Control.Monad (liftM)
 import Data.String (IsString (..))
@@ -94,15 +102,15 @@ instance Monoid a => Monoid (AForm sub master a) where
     mempty = pure mempty
     mappend a b = mappend <$> a <*> b
 
-data FieldSettings msg = FieldSettings
-    { fsLabel :: msg -- FIXME switch to SomeMessage?
-    , fsTooltip :: Maybe msg
+data FieldSettings master = FieldSettings
+    { fsLabel :: SomeMessage master
+    , fsTooltip :: Maybe (SomeMessage master)
     , fsId :: Maybe Text
     , fsName :: Maybe Text
-    , fsClass :: [Text]
+    , fsAttrs :: [(Text, Text)]
     }
 
-instance (a ~ Text) => IsString (FieldSettings a) where
+instance IsString (FieldSettings a) where
     fromString s = FieldSettings (fromString s) Nothing Nothing Nothing []
 
 data FieldView sub master = FieldView
@@ -116,10 +124,10 @@ data FieldView sub master = FieldView
 
 data Field sub master a = Field
     { fieldParse :: [Text] -> GHandler sub master (Either (SomeMessage master) (Maybe a))
-    -- | ID, name, class, (invalid text OR legimiate result), required?
+    -- | ID, name, attrs, (invalid text OR legimiate result), required?
     , fieldView :: Text
                 -> Text
-                -> [Text]
+                -> [(Text, Text)]
                 -> Either Text a
                 -> Bool
                 -> GWidget sub master ()
